unit NumEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,Clipbrd;

type
  TNumEdit = class(tedit)
  private
    FCheckRange: Boolean;
    FIntegersOnly: Boolean;
    FMin: Extended;
    FMax: Extended;
    FDisplayFormat: string;
    FFieldValue: Extended;
    FModified: Boolean;
    FOnInvalidPaste: TNotifyEvent;
    { Message Handling Methods }
    procedure CMEnter( var Msg: TCMEnter ); message cm_Enter;
    procedure CMExit( var Msg: TCMExit ); message cm_Exit;
    procedure WMPaste( var Msg: TWMPaste ); message wm_Paste;
    procedure WMCut( var Msg: TWMCut ); message wm_Cut;
  protected
    procedure CreateParams( var Params: TCreateParams ); override;
    function IsValidChar( Key: Char ): Boolean; virtual;
    function FormatText( Value: Extended ): string; virtual;
    function EvaluateText: Extended; virtual;
    { Event Dispatch Methods }
    procedure KeyPress( var Key: Char ); override;
    { Property Access Methods }
    procedure SetIntegersOnly( Value: Boolean ); virtual;

    procedure SetMin( Value: Extended ); virtual;
    procedure SetMax( Value: Extended ); virtual;

    function GetIntValue: Integer; virtual;
    procedure SetIntValue( Value: Integer ); virtual;
    function GetValue: Extended; virtual;
    function CheckValue( Value: Extended ): Extended; virtual;
    procedure SetValue( Value: Extended ); virtual;
    procedure SetDisplayFormat( FormatString: string ); virtual;
  public
    constructor Create( AOwner: TComponent ); override;

    property IntValue: Integer
      read GetIntValue
      write SetIntValue;

    property Modified: Boolean
      read FModified;

  published
    property CheckRange: Boolean
      read FCheckRange
      write FCheckRange
      default False;

    property IntegersOnly: Boolean
      read FIntegersOnly
      write SetIntegersOnly
      default True;

    property Max: Extended
      read FMax
      write SetMax;

    property Min: Extended
      read FMin
      write SetMin;

    property Value: Extended
      read GetValue
      write SetValue;

    property DisplayFormat: string
      read FDisplayFormat
      write SetDisplayFormat;

    property OnInvalidPasteOperate:TNotifyEvent
      read FOnInvalidPaste
      write FOnInvalidPaste;
  end;


implementation


{ TNumEdit }

function TNumEdit.CheckValue(Value: Extended): Extended;
begin
  Result := Value;
  if ( FMax <> FMin ) or FCheckRange then
  begin
    if Value < FMin then
        Result := FMin
    else if Value > FMax then
      Result := FMax;
  end;
  FModified := ( Result <> FFieldValue );
  FFieldValue := Result;
end;

procedure TNumEdit.CMEnter(var Msg: TCMEnter);
begin
  FModified := False;
  FFieldValue := EvaluateText;
  Text := FormatText( FFieldValue );
  inherited;
end;

procedure TNumEdit.CMExit(var Msg: TCMExit);
var
  N: Extended;
  MustSet: Boolean;
begin
  MustSet := False;
  try
    N := CheckValue( EvaluateText );
  except
    N := FMin;
    MustSet := True;
  end;
  if MustSet then
    SetValue( N );
  Text := FormatText( N );

  inherited;
end;


constructor TNumEdit.Create(AOwner: TComponent);
begin
  inherited Create( AOwner );
  ControlStyle := ControlStyle - [ csSetCaption ];
  Height := 21;
  Width := 65;
  FIntegersOnly := False;
  FCheckRange := False;
  FMin := 0;
  FMax := 0;
  FDisplayFormat := '#,##0,00';
  FFieldValue := 0.0;
  SetValue( FFieldValue );
  Text := FormatText( FFieldValue );
end;

procedure TNumEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams( Params );
  Params.Style := ( Params.Style or es_Right ) and not es_Multiline;
end;

function TNumEdit.EvaluateText: Extended;
var
  TmpText: string;
  Tmp: Byte;
  IsNeg: Boolean;
begin
  if ( Length( Text ) > 0 ) then
  begin
    IsNeg := ( Pos( '-', Text ) > 0 ) or ( Pos( '(', Text ) > 0 );
    TmpText := '';

    for Tmp := 1 to Length( Text ) do
    begin
      if Text[ Tmp ] in [ '0'..'9', DecimalSeparator ] then
        TmpText := TmpText + Text[ Tmp ];
    end;

    try
      if IsNeg then
        TmpText := '-' + TmpText;
      Result := StrToFloat( TmpText );
    except
      MessageBeep( 0 );
      Result := 0.0;
    end;
  end
  else
    Result := 0;
end;
function TNumEdit.FormatText(Value: Extended): string;
begin
  Result := FormatFloat( FDisplayFormat, Value );
end;

function TNumEdit.GetIntValue: Integer;
begin
  Result := Round( GetValue );
end;

function TNumEdit.GetValue: Extended;
begin
  try
    if Text = '' then
      Text := FormatText( FMin );
    Result := EvaluateText;
  except
    Result := FMin;
  end;
end;

function TNumEdit.IsValidChar(Key: Char): Boolean;
var
  ValidCharSet: set of Char;
begin
  if FIntegersOnly then
    ValidCharSet := [ '+', '-', '0'..'9' ]
  else
    ValidCharSet := [ DecimalSeparator, '+', '-', '0'..'9' ];

  Result := ( Key in ValidCharSet ) or ( ( Key < #32 )) ;
   //and ( Key <> Chr( vk_Return ) ) );

  if Result then
  begin
    if Key = DecimalSeparator then
    begin
      if SelLength = 0 then
        Result := Pos( DecimalSeparator, Text ) = 0
      else
      begin
        Result := Pos( DecimalSeparator, Text ) = 0;
        if not Result then
          Result := Pos( DecimalSeparator, SelText ) <> 0;
      end;
    end
    else if ( Key = '+' ) or ( Key = '-' ) then
      Result := ( ( SelStart = 0 ) and
                  ( Pos( '+', Text ) = 0 ) and
                  ( Pos( '-', Text ) = 0 ) ) or
                ( SelLength = Length( Text ) );
  end;
end;

procedure TNumEdit.KeyPress(var Key: Char);
var
  N: Extended;
  MustSet: Boolean;
begin
  if Key =#13 then begin
    MustSet := False;
    try
      N := CheckValue( EvaluateText );
    except
      N := FMin;
      MustSet := True;
    end;
    if MustSet then
      SetValue( N );
    Text := FormatText( N );
  end;

  inherited KeyPress( Key );

  if Key = #0 then Exit;

  if not IsValidChar( Key ) then
  begin
    Key := #0;
    MessageBeep( 0 );
  end;
end;

procedure TNumEdit.SetDisplayFormat(FormatString: string);
begin
  if FDisplayFormat <> FormatString then
  begin
    FDisplayFormat := FormatString;
    SetValue( Value );
    Text := FormatText( Value );
  end;
end;

procedure TNumEdit.SetIntegersOnly(Value: Boolean);
begin
  if FIntegersOnly <> Value then
  begin
    FIntegersOnly := Value;
    if FIntegersOnly then
    begin
      SetValue( Round( GetValue ) );
    end;
  end;
end;

procedure TNumEdit.SetIntValue(Value: Integer);
begin
  SetValue( Value );
end;

procedure TNumEdit.SetMax(Value: Extended);
begin
  if FMax <> Value then
  begin
    FMax := Value;
    if FMin > FMax then
      FMin := FMax;
    Invalidate;
  end;
end;

procedure TNumEdit.SetMin(Value: Extended);
begin
  if FMin <> Value then
  begin
    FMin := Value;
    if FMax < FMin then
      FMax := FMin;
    Invalidate;
  end;
end;

procedure TNumEdit.SetValue(Value: Extended);
begin
  if Value <> EvaluateText then
    Text := FormatText( Value );
end;

procedure TNumEdit.WMCut(var Msg: TWMCut);
begin
  if ReadOnly then
    Exit;
  inherited;
end;

procedure TNumEdit.WMPaste(var Msg: TWMPaste);
var
  str:String;
  i:integer;
begin
  if ReadOnly then Exit;
  if Clipboard.HasFormat(CF_TEXT) then begin
    str := Clipboard.AsText;
    for i := 0 to Length(str) do begin
      if not IsValidChar(str[i]) then begin
        if Assigned(FOnInvalidPaste) then FOnInvalidPaste(Self);
        //Application.MessageBox(pchar('аǷ֣ܽճ'),pchar(''),MB_OK);
        Exit;
      end;
    end;
  end;
  inherited;
end;

end.
